/* Keyboard example for Teensy USB Development Board
 * http://www.pjrc.com/teensy/usb_keyboard.html
 * Copyright (c) 2008 PJRC.COM, LLC
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "usb_keyboard.h"

#define LED_CONFIG	(DDRD |= (1<<6))
#define LED_ON		(PORTD &= ~(1<<6))
#define LED_OFF		(PORTD |= (1<<6))
#define CPU_PRESCALE(n)	(CLKPR = 0x80, CLKPR = (n))

#undef STR_MANUFACTURER
#undef STR_PRODUCT

#define STR_MANUFACTURER	L"Clayton G. Hobbs"
#define STR_PRODUCT		L"Music Score Foot Pedal"

uint16_t idle_count = 0;
uint8_t pressed = 0;
uint8_t last = 0;

int main(void)
{
	uint8_t d, mask;
	uint8_t d_prev=0xFF;

	// set for 16 MHz clock
	CPU_PRESCALE(0);

	// Configure all port B pins as inputs with pullup resistors.
	// See the "Using I/O Pins" page for details.
	// http://www.pjrc.com/teensy/pins.html
	DDRD = 0x00;
	PORTD = 0xFF;

	// Initialize the USB, and then wait for the host to set configuration.
	// If the Teensy is powered without a PC connected to the USB port,
	// this will wait forever.
	usb_init();
	while (!usb_configured())
        ;

	// Wait an extra second for the PC's operating system to load drivers
	// and do whatever it does to actually be ready for input
	_delay_ms(1000);

	// Configure timer 0 to generate a timer overflow interrupt every
	// 256*1024 clock cycles, or approx 61 Hz when using 16 MHz clock
	// This demonstrates how to use interrupts to implement a simple
	// inactivity timeout.
	TCCR0A = 0x00;
	TCCR0B = 0x05;
	TIMSK0 = (1<<TOIE0);

	while (1) {
		// read all port B and port D pins
		d = PIND;
		// check if any pins are low, but were high previously
		mask = 1 << 5;
		if (((d & mask) == 0) && (d_prev & mask) != 0) {
            cli();
            pressed += 1;
            idle_count = 0;
            sei();
		}
		// now the current pins will be the previous, and
		// wait a short delay so we're not highly sensitive
		// to mechanical "bounce".
		d_prev = d;
		_delay_ms(2);
	}
}

// This interrupt routine is run approx 61 times per second.
// All keypresses are sent here.  When a single press occurred with nothing
// else for a whole second, send a page down.  Further presses less than a
// second apart following that timeout send more page downs.  If after a period
// of inactivity, two presses occur within one second, send a page up.  Further
// presses less than a second apart send more page ups.
ISR(TIMER0_OVF_vect)
{
	idle_count++;
	if (idle_count > 30 && pressed > 0 && last != KEY_PAGE_UP) {
        usb_keyboard_press(KEY_PAGE_DOWN, 0);
        pressed -= 1;
		idle_count = 0;
        last = KEY_PAGE_DOWN;
	} else if (idle_count <= 30 && pressed == 2 && last != KEY_PAGE_DOWN) {
        usb_keyboard_press(KEY_PAGE_UP, 0);
        pressed -= 1;
        idle_count = 0;
        last = KEY_PAGE_UP;
    } else if (idle_count <= 30 && pressed > 0 && last == KEY_PAGE_DOWN) {
        usb_keyboard_press(KEY_PAGE_DOWN, 0);
        pressed -= 1;
		idle_count = 0;
        last = KEY_PAGE_DOWN;
    } else if (idle_count > 30 && pressed == 0) {
        last = 0;
        pressed = 0;
        idle_count = 0;
    } else if (idle_count > 30 && pressed > 0 && last == KEY_PAGE_UP) {
        last = 0;
        pressed = 0;
        idle_count = 0;
    }
}
